import os

import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.gridspec import GridSpec

from config import Config, FiguresConfig, DataConfig


def performance_vis():
    data = pd.read_csv(f"{Config.JaspOutputPath}/single_tasks.csv", sep=";")

    fig_size = (
        FiguresConfig.FullColumn,
        FiguresConfig.PageHeight * 0.2
    )
    margin = FiguresConfig.calc_margin_single(fig_size=fig_size, absolute_margin=(1.31, 1.05, 0.1, 0.875))
    spacing = FiguresConfig.calc_spacing_single(fig_size=fig_size, absolute_spacing=(0.1, 0.2))

    fig = plt.figure(
        figsize=fig_size,
        dpi=FiguresConfig.Dpi
    )
    gs = GridSpec(
        nrows=2,
        ncols=2,
        figure=fig,
        left=margin["left"],
        top=margin["top"],
        right=margin["right"],
        bottom=margin["bottom"],
        wspace=fig_size[0] * spacing["vertical"],
        hspace=fig_size[1] * spacing["horizontal"]
    )

    for (x, visKey) in enumerate(["Bar", "Scatter"]):
        for (y, dvKey) in enumerate(["Time", "Accuracy"]):
            ax = fig.add_subplot(gs[y, x])

            sns.regplot(
                ax=ax,
                data=data[(data["ChartType"] == visKey) & (data["Adaptation"] == "Adapted")],
                x=f"VL-{visKey}",
                y=f"{dvKey} {visKey}",
                scatter=FiguresConfig.Scatter,
                ci=95,
                x_jitter=.05,
                y_jitter=.05 if dvKey == "Accuracy" else 0,
                color=FiguresConfig.Palettes["Adaptation"]["Adapted"]
            )

            sns.regplot(
                ax=ax,
                data=data[(data["ChartType"] == visKey) & (data["Adaptation"] == "Non-Adapted")],
                x=f"VL-{visKey}",
                y=f"{dvKey} {visKey}",
                scatter=FiguresConfig.Scatter,
                ci=95,
                x_jitter=.05,
                y_jitter=.05 if dvKey == "Accuracy" else 0,
                color=FiguresConfig.Palettes["Adaptation"]["Non-Adapted"]
            )

            if y != 1:
                ax.set_xlabel("")
            else:
                ax.set_xlabel(FiguresConfig.AxisLabels[f"VL-{visKey}"])

            ylim = DataConfig.calc_fixed_value_limits(dvKey)
            ax.set_ylim(
                bottom=ylim[0],
                top=ylim[1]
            )
            ax.set_yticks(DataConfig.FixedValues[dvKey])
            ax.set_ylabel(FiguresConfig.AxisLabels[dvKey])

            for spine in ax.spines.values():
                spine.set_color(FiguresConfig.Palettes["ChartType"][visKey])
                spine.set_linewidth(FiguresConfig.BorderWidth)

    FiguresConfig.set_custom_legend(
        fig=fig,
        keys=["Bar Chart", "Scatter Plot"],
        colors=[
            FiguresConfig.Palettes["ChartType"]["Bar"],
            FiguresConfig.Palettes["ChartType"]["Scatter"],
        ],
        title="Visualization Type",
        loc="upper right"
    )
    FiguresConfig.set_custom_legend(
        fig=fig,
        keys=["Non-Adapted", "Adapted"],
        colors=[
            FiguresConfig.Palettes["Adaptation"]["Non-Adapted"],
            FiguresConfig.Palettes["Adaptation"]["Adapted"],
        ],
        title="Adaptation",
        loc="upper left"
    )

    fig.align_ylabels()
    fig.tight_layout()
    for format_str in FiguresConfig.VisFormats:
        fig.savefig(f"{Config.OtherContentOutputPath}/performance vis.{format_str}")
    plt.close(fig)


def subjective_vis():
    data = pd.read_csv(f"{Config.JaspOutputPath}/aggregated_tasks.csv", sep=";")

    fig_size = (
        FiguresConfig.FullColumn,
        FiguresConfig.PageHeight * 0.3
    )
    margin = FiguresConfig.calc_margin_single(fig_size=fig_size, absolute_margin=(1.31, 1.05, 0.1, 0.875))
    spacing = FiguresConfig.calc_spacing_single(fig_size=fig_size, absolute_spacing=(0.1, 0.25))

    fig = plt.figure(
        figsize=fig_size,
        dpi=FiguresConfig.Dpi
    )
    gs = GridSpec(
        nrows=3,
        ncols=2,
        figure=fig,
        left=margin["left"],
        top=margin["top"],
        right=margin["right"],
        bottom=margin["bottom"],
        wspace=fig_size[0] * spacing["vertical"],
        hspace=fig_size[1] * spacing["horizontal"]
    )

    for (x, visKey) in enumerate(["Bar", "Scatter"]):
        for (y, dvKey) in enumerate(["Dependability", "Usefulness", "Intuitive Use"]):
            ax = fig.add_subplot(gs[y, x])

            sns.regplot(
                ax=ax,
                data=data[(data["ChartType"] == visKey) & (data["Adaptation"] == "Adapted")],
                x=f"VL-{visKey}",
                y=f"{dvKey} {visKey}",
                scatter=FiguresConfig.Scatter,
                ci=95,
                x_jitter=.05,
                y_jitter=.05 if dvKey == "Accuracy" else 0,
                color=FiguresConfig.Palettes["Adaptation"]["Adapted"]
            )

            sns.regplot(
                ax=ax,
                data=data[(data["ChartType"] == visKey) & (data["Adaptation"] == "Non-Adapted")],
                x=f"VL-{visKey}",
                y=f"{dvKey} {visKey}",
                scatter=FiguresConfig.Scatter,
                ci=95,
                x_jitter=.05,
                y_jitter=.05 if dvKey == "Accuracy" else 0,
                color=FiguresConfig.Palettes["Adaptation"]["Non-Adapted"]
            )

            if y != 2:
                ax.set_xlabel("")
            else:
                ax.set_xlabel(FiguresConfig.AxisLabels[f"VL-{visKey}"])

            ylim = DataConfig.calc_fixed_value_limits(dvKey)
            ax.set_ylim(
                bottom=ylim[0],
                top=ylim[1]
            )
            ax.set_yticks(DataConfig.FixedValues[dvKey])
            ax.set_ylabel(FiguresConfig.AxisLabels[dvKey])

            ylim = DataConfig.calc_fixed_value_limits(dvKey)
            ax.set_ylim(
                bottom=ylim[0],
                top=ylim[1]
            )
            ax.set_yticks(DataConfig.FixedValues[dvKey])

            for spine in ax.spines.values():
                spine.set_color(FiguresConfig.Palettes["ChartType"][visKey])
                spine.set_linewidth(FiguresConfig.BorderWidth)

    FiguresConfig.set_custom_legend(
        fig=fig,
        keys=["Bar Chart", "Scatter Plot"],
        colors=[
            FiguresConfig.Palettes["ChartType"]["Bar"],
            FiguresConfig.Palettes["ChartType"]["Scatter"],
        ],
        title="Visualization Type",
        loc="upper right"
    )
    FiguresConfig.set_custom_legend(
        fig=fig,
        keys=["Non-Adapted", "Adapted"],
        colors=[
            FiguresConfig.Palettes["Adaptation"]["Non-Adapted"],
            FiguresConfig.Palettes["Adaptation"]["Adapted"],
        ],
        title="Adaptation",
        loc="upper left"
    )

    fig.align_ylabels()
    fig.tight_layout()
    for format_str in FiguresConfig.VisFormats:
        fig.savefig(f"{Config.OtherContentOutputPath}/subjective vis.{format_str}")
    plt.close(fig)


if __name__ == '__main__':
    FiguresConfig.set_rc_plot_values()

    with sns.axes_style(FiguresConfig.SnsStyle):
        performance_vis()
        subjective_vis()
